function[quarterlyseries_avg, monthlyseries_avg,dailyseries,quarterlyseries_eop,monthlyseries_eop]=d2qm(dailyseries_raw)
%function[monthlyseries,dailyseries]=d2qm(dailyseries_raw)

% d2qm.m transforms a daily series into quarterly and monthly series, in quarterly/monthly averages and end of period. 
% dailyseries_raw is an n_rawday-by-(n_var+4) matrix with col. 1-4 being year, quarter, month and day, 
% missing value denoted as -9999; the output monthlyseries is an n_day-by-(n_var+2) matrix with col. 1-2 the year and quarter.

n_rawday = size(dailyseries_raw,1); % number of raw daily observations. missing value denoted as -9999.
n_var    = size(dailyseries_raw,2)-4;

dailyseries = zeros(n_rawday,n_var+4);

n_day    = 0;
for i = 1:n_rawday;
    if prod(dailyseries_raw(i,5:end)~=-9999) ==1;   % if none of the variables have missing value.
        n_day = n_day+1;
        dailyseries(n_day,:) = dailyseries_raw(i,:);
    end
end
dailyseries = dailyseries(1:n_day,:);

monthlyseries_avg = zeros(floor(n_day/25),n_var+2);
monthlyseries_eop = zeros(floor(n_day/25),n_var+2);
n_m   = 1;      % number of months.
monthlyseries_avg(1,1:2) = [dailyseries(1,1),dailyseries(1,3)]; % The first month.
monthlyseries_eop(1,1:2) = [dailyseries(1,1),dailyseries(1,3)]; % The first month.
for i_m = 2:31;
    if (dailyseries(i_m,1)==dailyseries(i_m-1,1)&&dailyseries(i_m,3)==dailyseries(i_m-1,3))==0;
        break;
    end
end
monthlyseries_avg(1,3:end) = mean(dailyseries(1:i_m-1,5:end));  % the first monthly observation.
monthlyseries_eop(1,3:end) = dailyseries(i_m-1,5:end);  % the first monthly end-of-period observation.
ind_d = i_m;      % indicator of the beginning of the next month.

for j_m = ind_d:n_day;
    if (dailyseries(j_m,1)==dailyseries(j_m-1,1)&&dailyseries(j_m,3)==dailyseries(j_m-1,3))==0;
        n_m = n_m+1;
        monthlyseries_avg(n_m,1:2) = [dailyseries(j_m,1),dailyseries(j_m,3)];  % a new monthly observation.
        monthlyseries_eop(n_m,1:2) = [dailyseries(j_m,1),dailyseries(j_m,3)];  % a new monthly observation.
        ind_endtype = 0;        % ind_endtype = 1 if the loop breaks at the beginning of next month, = 0  if no break and k will equal n_day.
        for k_m = j_m+1:n_day;
            if (dailyseries(k_m,1)==dailyseries(k_m-1,1)&&dailyseries(k_m,3)==dailyseries(k_m-1,3))==0;
                ind_endtype = 1;        
                break;
            end
        end
        if ind_endtype ==1;
            monthlyseries_avg(n_m,3:end) = mean(dailyseries(j_m:k_m-1,5:end));
            monthlyseries_eop(n_m,3:end) = dailyseries(k_m-1,5:end);
            ind_d = k_m;
        else
            monthlyseries_avg(n_m,3:end) = mean(dailyseries(j_m:k_m,5:end));    % the last monthly observation.
            monthlyseries_eop(n_m,3:end) = dailyseries(k_m,5:end);    
        end
    end
end

quarterlyseries_avg = zeros(floor(n_day/85),n_var+2);
quarterlyseries_eop = zeros(floor(n_day/85),n_var+2);
n_q   = 1;      % number of quarters.
quarterlyseries_avg(1,1:2) = [dailyseries(1,1),dailyseries(1,2)]; % The first quarter.
quarterlyseries_eop(1,1:2) = [dailyseries(1,1),dailyseries(1,2)]; % The first quarter.
for i_q = 2:95;
    if (dailyseries(i_q,1)==dailyseries(i_q-1,1)&&dailyseries(i_q,2)==dailyseries(i_q-1,2))==0;
        break;
    end
end
quarterlyseries_avg(1,3:end) = mean(dailyseries(1:i_q-1,5:end));  % the first quarterly observation.
quarterlyseries_eop(1,3:end) = dailyseries(i_q-1,5:end);  % the first quarterly observation.
ind_d = i_q;      % indicator of the beginning of the next quarter.

for j_q = ind_d:n_day;
    if (dailyseries(j_q,1)==dailyseries(j_q-1,1)&&dailyseries(j_q,2)==dailyseries(j_q-1,2))==0;
        n_q = n_q+1;
        quarterlyseries_avg(n_q,1:2) = [dailyseries(j_q,1),dailyseries(j_q,2)];  % a new quarterly observation.
        quarterlyseries_eop(n_q,1:2) = [dailyseries(j_q,1),dailyseries(j_q,2)];  % a new quarterly observation.
        ind_endtype = 0;        % ind_endtype = 1 if the loop breaks at the beginning of next quarter, = 0  if no break and k will equal n_day.
        for k_q = j_q+1:n_day;
            if (dailyseries(k_q,1)==dailyseries(k_q-1,1)&&dailyseries(k_q,2)==dailyseries(k_q-1,2))==0;
                ind_endtype = 1;        
                break;
            end
        end
        if ind_endtype ==1;
            quarterlyseries_avg(n_q,3:end) = mean(dailyseries(j_q:k_q-1,5:end));
            quarterlyseries_eop(n_q,3:end) = dailyseries(k_q-1,5:end);
            ind_d = k_q;
        else
            quarterlyseries_avg(n_q,3:end) = mean(dailyseries(j_q:k_q,5:end));    % the last quarterly observation.
            quarterlyseries_eop(n_q,3:end) = dailyseries(k_q,5:end);    % the last quarterly observation.
        end
    end
end