function[F_star]=fstar_kvhac(yt_all,xt_all,R,r);
% fstar_kvhac.m computes the F_star statistic in the F_star test from Kiefer and Vogelsang (2002): H0: R*beta = r,  
% where beta (m-by-1) is from the OLS regression yt_all = xt_all*beta+u,  and R (q-by-m) and r (q-by-1) are linear constraints to be tested.
% yt_all is T-by-1 vector and xt_all is T-by-m matrix.

[T,m]   = size(xt_all);

beta    = inv(xt_all'*xt_all)*xt_all'*yt_all;
ut_all  = yt_all-xt_all*beta;   % residuals

st_all = zeros(T,m);
st_all(1,:) = xt_all(1,:)*ut_all(1);
for t = 2:T;
    st_all(t,:) = st_all(t-1,:)+xt_all(t,:)*ut_all(t);
end

% C = st_all(1,:)'*st_all(1,:);
% for t = 2:T;
%     C = C+st_all(t,:)'*st_all(t,:);
% end
%%% This is identical to C = st_all'*st_all below. 

C = st_all'*st_all/T^2;

%asyvar = inv(xt_all'*xt_all/T)*2*C*inv(xt_all'*xt_all/T)/T;     % Note: Kiefer-Vogelsang (2002) show that omiga = 2*C, and then construct t_star stat using omiga in their formula. 
asyvar = inv(xt_all'*xt_all)*2*C*inv(xt_all'*xt_all)*T;  

tstat = beta./sqrt(diag(asyvar));

q = size(R,1);  % number of constraints.
%F_star = T*(R*beta-r)'*inv(R*inv(xt_all'*xt_all/T)*2*C*inv(xt_all'*xt_all/T)*R')*(R*beta-r)/q;
F_star = 1/T*(R*beta-r)'*inv(R*inv(xt_all'*xt_all)*2*C*inv(xt_all'*xt_all)*R')*(R*beta-r)/q;