function[beta,tstat]=ols_kvhac(yt_all,xt_all);
% ols_kvhac.m computes the OLS regression of y = x*beta+u, and reports the Kiefer-Vogelsang-Bunzel (Econometrica 2000)  
% heteroskedasticity- and autocorrelation-consistant t statistic for the OLS estimator.
% yt_all is T-by-1 vector and xt_all is T-by-m matrix.
% As Kiefer-Vogelsan (Econometria 2002) display, this test is equivalent to using Bartlett kernel HAC (Newey-West) with bandwidth equal to 
% sample size. 

[T,m]   = size(xt_all);

beta    = inv(xt_all'*xt_all)*xt_all'*yt_all;
ut_all  = yt_all-xt_all*beta;   % residuals

st_all = zeros(T,m);
st_all(1,:) = xt_all(1,:)*ut_all(1);
for t = 2:T;
    st_all(t,:) = st_all(t-1,:)+xt_all(t,:)*ut_all(t);
end

% C = st_all(1,:)'*st_all(1,:);
% for t = 2:T;
%     C = C+st_all(t,:)'*st_all(t,:);
% end
%%% This is identical to C = st_all'*st_all below. 

C = st_all'*st_all/T^2;

%asyvar = inv(xt_all'*xt_all/T)*2*C*inv(xt_all'*xt_all/T)/T;     % Note: Kiefer-Vogelsang (2002) show that omiga = 2*C, and then construct t_star stat using omiga in their formula. 
asyvar = inv(xt_all'*xt_all)*2*C*inv(xt_all'*xt_all)*T;  

tstat = beta./sqrt(diag(asyvar));